@@ -2,6 +2,7 @@
 Miscillaneous functions...
 """
 import os,stat,sys,numpy as np
+import pkg_resources
 import starbug2
 from starbug2.utils import *
 from starbug2.matching import sort_exposures
@@ -114,3 +115,51 @@ def generate_runscript(fnames, args="starbug2 "):
     os.chmod(RUNFILE,stat.S_IXUSR|stat.S_IWUSR|stat.S_IRUSR|stat.S_IRGRP|stat.S_IROTH)
     printf("->%s\n"%RUNFILE)
 
+
+def update_paramfile(fname):
+    """
+    When the local parameter file is from an older version, add or remove the
+    new or obselete keys
+    INPUT: fname=local file to update
+    """
+    default_fname="%s/default.param"%pkg_resources.resource_filename("starbug2","param/")
+    default_param=load_params(default_fname)
+    current_param=load_params(fname)
+
+    if default_fname==fname:
+        #perror("cannot change default parameter file")
+        return 
+
+    if os.path.exists(fname):
+        printf("Updating \"%s\"\n"%fname)
+        fpi=open(fname, 'r')
+        fpd=open(default_fname, 'r')
+        fpo=open("/tmp/starbug.param",'w')
+
+        add_keys=set(default_param.keys())-set(current_param.keys())
+        del_keys=set(current_param.keys())-set(default_param.keys())
+        if add_keys: printf("-> adding: %s  \n"%(', '.join(add_keys)))
+        if del_keys: printf("-> removing: %s\n"%(', '.join(del_keys)))
+        
+        if not len(add_keys|del_keys): 
+            printf("-> No updates needed\n")
+            return 
+
+        for inline in fpd.readlines():
+            if inline[0] in "# \t\n":
+                fpo.write(inline)
+                continue
+
+            key,value,comment=parse("{}={}//{}\n",inline)
+            key=key.strip().rstrip()
+
+            if key not in add_keys:
+                value=current_param[key]
+
+            outline="%-24s"%("%-12s"%key+"= "+str(value))+" //"+comment+"\n"
+            fpo.write(outline)
+
+        os.system("mv /tmp/starbug.param %s"%fname)
+    else: perror("local parameter file '%s' does not exist\n"%fname)
+
+
